__author__ = 'mnowotka'

#-----------------------------------------------------------------------------------------------------------------------

import StringIO
from rdkit.Chem.Draw import SimilarityMaps
from chembl_beaker.beaker.utils.io import _parseSMILESData, _parseMolData

try:
    import matplotlib
    matplotlib.use('Agg')
    from matplotlib import pyplot
except:
    matplotlib=None

#-----------------------------------------------------------------------------------------------------------------------

def _similarityMap(ms,params):
    if matplotlib is None:
        raise ValueError('matplotlib not useable')

    fp=params.get('fingerprint','morgan')
    if fp=='morgan':
        rad = int(params.get('radius',2))
        fn = lambda x,i:SimilarityMaps.GetMorganFingerprint(x,i,radius=rad)
    elif fp=='tt':
        fn = SimilarityMaps.GetAPFingerprint
    elif fp=='ap':
        fn = SimilarityMaps.GetTTFingerprint

    w = int(params.get('w',100))
    h = int(params.get('h',100))

    fig,maxv = SimilarityMaps.GetSimilarityMapForFingerprint(ms[0],ms[1],fn,size=(w,h))
    sio = StringIO.StringIO()
    pyplot.savefig(sio,format='png',bbox_inches='tight',dpi=100)

    return sio.getvalue()

#-----------------------------------------------------------------------------------------------------------------------

def _smiles2SimilarityMap(data,params):
    return _similarityMap(_parseSMILESData(data, True), params)

#-----------------------------------------------------------------------------------------------------------------------

def _sdf2SimilarityMap(data,params):
    return _similarityMap(_parseMolData(data), params)

#-----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

#-----------------------------------------------------------------------------------------------------------------------

from bottle import request
import base64
import os
from chembl_beaker.beaker import app, config
from chembl_beaker.beaker.core_apps.osra.impl import _image2ctab

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/image2ctab/<image>', method=['OPTIONS', 'GET'], name="image2ctab")
def image2ctab(image):
    """
Uses OSRA to convert image to CTAB. Image should be urlsafe_base65 encoded data of 300 DPI png graphic.
    """

    if image.startswith('data:'):
        img = base64.urlsafe_b64decode(image[image.find(',')+1:])
    else:
        img = base64.urlsafe_b64decode(image)
    #TODO: check /usr/local/bin/osra when no explicit value given
    known_location = '/usr/local/bin/osra'
    if not os.path.exists(known_location):
        known_location = '/usr/bin/osra'
    return _image2ctab(img, config.get('osra_binaries_location', known_location))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/image2ctab', method=['OPTIONS', 'POST'], name="image2ctab")
def image2ctab():
    """
Uses OSRA to convert image to CTAB. Image should be 300 DPI png graphic.
    """

    img = request.body.read()
    if img.startswith('data:'):
        img = base64.b64decode(img[img.find(',')+1:])
    #TODO: check /usr/local/bin/osra when no explicit value given
    known_location = '/usr/local/bin/osra'
    if not os.path.exists(known_location):
        known_location = '/usr/bin/osra'
    return _image2ctab(img, config.get('osra_binaries_location', known_location))

#-----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

#-----------------------------------------------------------------------------------------------------------------------

from chembl_beaker.beaker import app
from chembl_beaker.beaker.core_apps.jsonImages.impl import _ctab2json, _smiles2json
from bottle import request, response
import base64

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/ctab2json/<ctab>', method=['OPTIONS', 'GET'], name="ctab2json")
@app.route('/ctab2json/<ctab>/<size>', method=['OPTIONS', 'GET'], name="ctab2json")
@app.route('/ctab2json/<ctab>/<size>/<legend>', method=['OPTIONS', 'GET'], name="ctab2json")
def ctab2json(ctab, size=200, legend=''):
    """
Converts CTAB to JSON. Resulting JSON contains data about 2D graphic representation of compounds and can be used
directly by Raphael.js library to draw compound image in browser. CTAB is urlsafe_base64 encoded string containing
single molfile or concatenation of multiple molfiles. Size is the optional size of image in pixels (default value
is 200 px). Legend is optional label in the bottom of image.
    """

    data = base64.urlsafe_b64decode(ctab)
    response.content_type = 'application/json'
    return _ctab2json(data, size, legend)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/ctab2json', method=['OPTIONS', 'POST'], name="ctab2json")
def ctab2json():
    """
Converts CTAB to JSON. Resulting JSON contains data about 2D graphic representation of compounds and can be used
directly by Raphael.js library to draw compound image in browser. CTAB is either single molfile or SDF file. Size
is the optional size of image in pixels (default value is 200 px). Legend is optional label in the bottom of image.
    """

    size = int(request.forms.get('size', 200))
    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    legend=request.params.get('legend','')
    response.content_type = 'application/json'
    return _ctab2json(data, size, legend)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/smiles2json/<smiles>', method=['OPTIONS', 'GET'], name="smiles2json")
@app.route('/smiles2json/<smiles>/<size>', method=['OPTIONS', 'GET'], name="smiles2json")
@app.route('/smiles2json/<smiles>/<size>/<legend>', method=['OPTIONS', 'GET'], name="smiles2json")
def smiles2json(smiles, size=200, legend=''):
    """
Converts SMILES to JSON. Resulting JSON contains data about 2D graphic representation of compounds and can be used
directly by Raphael.js library to draw compound image in browser. This method accepts urlsafe_base64 encoded string
containing single or multiple SMILES optionally containing header line, specific to *.smi format. Size is the
optional size of image in pixels (default value is 200 px). Legend is optional label in the bottom of image.
    """

    data = base64.urlsafe_b64decode(smiles)
    response.content_type = 'application/json'
    return _smiles2json(data, size, legend)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/smiles2json', method=['OPTIONS', 'POST'], name="smiles2json")
def smiles2json():
    """
Converts SMILES to JSON. Resulting JSON contains data about 2D graphic representation of compounds and can be used
directly by Raphael.js library to draw compound image in browser. This method accepts single or multiple SMILES or
*.smi file. Size is the optional size of image in pixels (default valueis 200 px). Legend is optional label in
the bottom of image.
    """

    size = int(request.forms.get('size', 200))
    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    legend=request.params.get('legend','')
    response.content_type = 'application/json'
    return _smiles2json(data, size, legend)

#-----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

from lxml.etree import _ElementTree, Element, tostring
from lxml import objectify
from lxml.objectify import ObjectifiedElement, StringElement
import json
from chembl_beaker import __version__ as version
from rdkit import Chem
from StringIO import StringIO
import os

MOL_MARVIN_SCALE = 1.8666666517333335

molBondTypes = {
"SINGLE" : 1,
"DOUBLE" : 2,
"TRIPLE" : 3,
"AROMATIC" : 4,
"UNSPECIFIED" : 8,
}

#-----------------------------------------------------------------------------------------------------------------------

class MarvinJSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if type(obj) == _ElementTree:
            return obj.getroot()
        if type(obj) == ObjectifiedElement:
            if obj.tag == 'cml':
                return  obj.find('MDocument')
            if obj.tag == 'MDocument':
                return [ el for el in obj.iterchildren(tag='MChemicalStruct') ]
            if obj.tag == 'MChemicalStruct':
                return obj.find('molecule')
            if obj.tag == 'molecule':
                return {obj.get("molID"): {"atoms" : obj.find("atomArray"), "bonds": obj.find("bondArray")}}
            if obj.tag == 'bondArray':
                return [ bond for bond in obj.iterchildren(tag='bond') ]
        if type(obj) == StringElement:
            if obj.tag == 'atomArray':
                ids = obj.get('atomID').split()
                num_atoms = len(ids)
                zeros = [0] * num_atoms

                if obj.get('x2', False):
                    return {"atomID": ids,
                            "x": [float(x) for x in obj.get('x2').split()],
                            'y': [float(y) for y in obj.get('y2').split()],
                            'z': zeros,
                            'elementType': obj.get('elementType').split()}

                else:
                    return {"atomID": ids,
                            "x": [float(x) for x in obj.get('x3').split()],
                            'y': [float(y) for y in obj.get('y3').split()],
                            'z': [float(y) for y in obj.get('z3').split()],
                            'elementType': obj.get('elementType').split()}

            if obj.tag == 'bond':
                return {"atomRefs" : obj.get('atomRefs2').split(), "order": int(obj.get('order'))}

        return json.JSONEncoder.default(self, obj)

#-----------------------------------------------------------------------------------------------------------------------

def _dict2Mol(obj, scale = 1.0):
    buffer = StringIO()
    buffer.write('\nConverted by chembl_beaker ver. %s\n\n' % version)
    num_atoms = len(obj['atoms']['atomID'])
    zeros = [0] * num_atoms
    buffer.write('{:3}{:3}{:3}{:3}{:3}{:3}          {} V2000\n'.format(
                                                            num_atoms, len(obj['bonds']), 0, 0, 0, 0, 1))
    for i, atom in enumerate(obj['atoms']['elementType']):
        buffer.write('{:10,.4f}{:10,.4f}{:10,.4f} {:<2}{:3}{:3}\n'.format(
                      obj['atoms'].get('x', zeros)[i] / scale,
                      obj['atoms'].get('y', zeros)[i] / scale,
                      obj['atoms'].get('z', zeros)[i] / scale,
                      atom, 0, 0 ))
    for bond in obj['bonds']:
        first_atom = None
        second_atom = None
        first_id, second_id = bond['atomRefs']
        for i, atomID in enumerate(obj['atoms']['atomID']):
            if first_id == atomID:
                first_atom = i + 1
            if second_id == atomID:
                second_atom = i + 1
            if first_atom is not None and second_atom is not None:
                break
        buffer.write('{:3}{:3}{:3}{:3}\n'.format(first_atom, second_atom, bond['order'], 0))
    buffer.write('M  END\n')
    return buffer.getvalue()

#-----------------------------------------------------------------------------------------------------------------------

def _jsonToMol(obj, scale = 1.0):
    return '$$$$\n'.join([_dict2Mol(x.values()[0], scale) for x in obj])

#-----------------------------------------------------------------------------------------------------------------------

def _molToJson(mol, scale = 1.0):
    atoms = {"atomID":[], "elementType":[], "x":[], "y":[]}
    conformer = mol.GetConformer()
    is3D = conformer.Is3D()
    if is3D:
        atoms['z'] = []
    for atom in mol.GetAtoms():
        idx = atom.GetIdx()
        point = conformer.GetAtomPosition(idx)
        atoms['x'].append(point.x * scale)
        atoms['y'].append(point.y * scale)
        if is3D:
            atoms['z'].append(point.y * scale)
        atoms["atomID"].append("a%s" % (idx + 1))
        atoms["elementType"].append(atom.GetSymbol())
    bonds = []
    for bond in mol.GetBonds():
        atomRefs = ["a%s" % (bond.GetBeginAtomIdx() + 1), "a%s" % (bond.GetEndAtomIdx() + 1)]
        order = molBondTypes[str(bond.GetBondType())]
        bonds.append({"atomRefs": atomRefs, "order":order})
    return {"atoms":atoms, "bonds":bonds}

#-----------------------------------------------------------------------------------------------------------------------

def _molsToJson(mols, scale = 1.0):
    ret = []
    for idx, mol in enumerate(mols):
        ret.append({"m%s" % (idx + 1): _molToJson(mol, scale)})
    return ret

#-----------------------------------------------------------------------------------------------------------------------

def _dictToEtree(data, name=None, depth=0):
    element = None
    if depth == 0:
        element = Element('cml')
        element.append(_dictToEtree(data, name, depth + 1))
    elif depth == 1:
        element = Element('MDocument')
        element.append(_dictToEtree(data, name, depth + 1))
    elif depth == 2:
        element = Element('MChemicalStruct')
        for mol in data:
            element.append(_dictToEtree(mol, name, depth + 1))
    elif depth == 3:
        molID = data.keys()[0]
        val = data.values()[0]
        element = Element('molecule', molID=molID)
        element.append(_dictToEtree(val['atoms'], 'atoms', depth + 1))
        element.append(_dictToEtree(val['bonds'], 'bonds', depth + 1))
    elif depth == 4:
        if name == 'atoms':
            kwargs = {}
            kwargs['atomID'] = ' '.join(data['atomID'])
            kwargs['elementType'] = ' '.join(data['elementType'])
            if 'z' not in data:
                kwargs['x2'] = ' '.join("%.15f" % x for x in data['x'])
                kwargs['y2'] = ' '.join("%.15f" % y for y in data['y'])
            else:
                kwargs['x3'] = ' '.join("%.15f" % x for x in data['x'])
                kwargs['y3'] = ' '.join("%.15f" % y for y in data['y'])
                kwargs['z3'] = ' '.join("%.15f" % z for z in data['z'])

            element = Element('atomArray', **kwargs)
        elif name == 'bonds':
            element = Element('bondArray')
            for bond in data:
                element.append(_dictToEtree(bond, 'bond', depth + 1))
    elif depth == 5:
        kwargs = {}
        kwargs['atomRefs2'] = ' '.join(data['atomRefs'])
        kwargs['order'] = str(data['order'])
        element = Element('bond', **kwargs)
    return element

#-----------------------------------------------------------------------------------------------------------------------

def _dataToXml(data, options=None):
    options = options or {}
    return tostring(_dictToEtree(data, options), xml_declaration=False, encoding='utf-8')

#-----------------------------------------------------------------------------------------------------------------------

def MolToMarvin(mol):
    if mol.endswith('.mol') and os.path.exists(mol):
        mol = Chem.MolFromMolFile(mol, False, False, False)
    else:
        mol = Chem.MolFromMolBlock(mol, False, False, False)
    js = _molsToJson([mol], MOL_MARVIN_SCALE)
    return _dataToXml(js)

#-----------------------------------------------------------------------------------------------------------------------

def SDFToMarvin(mol):
    mols = []
    if mol.endswith('.sdf') and os.path.exists(mol):
        suppl = Chem.SDMolSupplier('mol')
        mols = [mol for mol in suppl]

    js = _molsToJson(mols, MOL_MARVIN_SCALE)
    return _dataToXml(js)

#-----------------------------------------------------------------------------------------------------------------------

def MarvinToMol(marvin):
    if marvin.endswith('.mrv') and os.path.exists(marvin):
        f = open(marvin)
    else:
        f = StringIO(marvin)
    tree = objectify.parse(f)
    js = MarvinJSONEncoder().encode(tree)
    return _jsonToMol(json.loads(js), MOL_MARVIN_SCALE)

#-----------------------------------------------------------------------------------------------------------------------


__author__ = 'mnowotka'

#-----------------------------------------------------------------------------------------------------------------------

from chembl_beaker.beaker import app
from bottle import request, response
from chembl_beaker.beaker.core_apps.D3Coords.impl import _ctab23D, _smiles23D
import base64

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/ctab23D/<ctab>', method=['OPTIONS', 'GET'], name="ctab23D")
@app.route('/ctab23D/<ctab>/<multi>', method=['OPTIONS', 'GET'], name="ctab23D")
def ctab23D(ctab, multi=False):
    """
Generate 3D coordinates from molfile using Universal Force Field.
CTAB is urlsafe_base64 encoded string containing single molfile or concatenation of multiple molfiles.
    """

    data = base64.urlsafe_b64decode(ctab)
    return _ctab23D(data, int(multi))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/ctab23D', method=['OPTIONS', 'POST'], name="ctab23D")
def ctab23D():
    """
Generate 3D coordinates from molfile using Universal Force Field.
CTAB is either single molfile or SDF file.
    """

    multi = int(request.forms.get('multi', False))
    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return _ctab23D(data, multi)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/smiles23D/<ctab>', method=['OPTIONS', 'GET'], name="smiles23D")
@app.route('/smiles23D/<ctab>/<multi>', method=['OPTIONS', 'GET'], name="smiles23D")
def smiles23D(ctab, multi=False):
    """
Generate 3D coordinates from SMILES using Universal Force Field.
CTAB is urlsafe_base64 encoded string containing single molfile or concatenation of multiple molfiles.
    """

    data = base64.urlsafe_b64decode(ctab)
    return _smiles23D(data, int(multi))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/smiles23D', method=['OPTIONS', 'POST'], name="smiles23D")
def smiles23D():
    """
Generate 3D coordinates from SMILES using Universal Force Field.
CTAB is either single molfile or SDF file.
    """

    multi = int(request.forms.get('multi', False))
    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return _smiles23D(data, multi)

#-----------------------------------------------------------------------------------------------------------------------


__author__ = 'mnowotka'

from bottle import request, response
import json

from chembl_beaker.beaker import app
from chembl_beaker.beaker.core_apps.marvin.impl import _clean2D, _stereoInfo, _molExport

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/clean', method=['OPTIONS', 'POST'], name="clean")
def clean2D():
    """
Implements Marvin 4 js clean2D web service. Recomputes 2D coordinates of given compound. The compound is in
Marvin's *.mrv format.
    """

    params = json.loads(request.body.read())
    structure = params['structure']
    response.content_type = 'text/plain'
    return _clean2D(structure)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/cipStereoInfo', method=['OPTIONS', 'POST'], name="cipStereoInfo")
def stereoInfo():
    """
Implements Marvin 4 js Stereo Info web service. Marks possible stereocenters, R/S chirality for atoms and cistrans
bonds.
    """

    params = json.loads(request.body.read())
    structure = params['structure']
    response.content_type = 'application/json'
    return json.dumps(_stereoInfo(structure))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/molExport' , method=['OPTIONS', 'POST'], name="molExport")
def molExport():
    """
Implements Marvin 4 js MolConvert web service. Converts compound from one format to another, including Marvin's
*.mrv.
    """

    params = json.loads(request.body.read())
    structure = params['structure']
    input_f = params['inputFormat']
    output_f = params['parameters']

    res = _molExport(structure, input_f, output_f)
    response.content_type = 'application/json'
    return json.dumps(res)

#-----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

#-----------------------------------------------------------------------------------------------------------------------

import StringIO
from rdkit import rdBase
from rdkit import Chem
from rdkit import DataStructs
from rdkit.Chem.AtomPairs import Pairs
from rdkit.Chem import MACCSkeys
from rdkit.Chem import rdMolDescriptors

from chembl_beaker.beaker.utils.io import _parseMolData

#-----------------------------------------------------------------------------------------------------------------------

def _getFPSStream(f, mols, type='morgan', radius=2, n_bits=2048):
    f.write("#FPS1\n#num_bits=%s\n#software=RDKit/%s\n" % (n_bits, rdBase.rdkitVersion))
    for i, mol in enumerate(mols):
        if mol:
            idx = i
            if mol.HasProp('chembl_id'):
                idx = mol.GetProp('chembl_id')
            else:
                try:
                    idx = Chem.InchiToInchiKey(Chem.MolToInchi(mol))
                except:
                    pass
            if type == 'morgan':
                fp = rdMolDescriptors.GetMorganFingerprintAsBitVect(mol,radius,nBits=n_bits)
            elif type == 'pair':
                fp = Pairs.GetAtomPairFingerprintAsBitVect(mol)
            elif type == 'maccs':
                fp = MACCSkeys.GenMACCSKeys(mol)
            f.write("%s\t%s\n" % (DataStructs.BitVectToFPSText(fp), idx))

#-----------------------------------------------------------------------------------------------------------------------

def _getFPSString(mols, type='morgan', radius=2, n_bits=2048):
    sio = StringIO.StringIO()
    _getFPSStream(sio, mols, type, radius, n_bits)
    return sio.getvalue()

#-----------------------------------------------------------------------------------------------------------------------

def _sdf2fps(sdf, type='morgan', radius=2, n_bits=2048):
    return _getFPSString(_parseMolData(sdf), type, radius, n_bits)

#-----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

import bottle
from bottle import Bottle
import chembl_beaker
import re
import os
import sys
import json

#-----------------------------------------------------------------------------------------------------------------------

HTTP_CODES = bottle.HTTP_CODES.copy()
HTTP_CODES = dict((y,x) for x,y in HTTP_CODES.iteritems())
STATIC_ROOT = os.path.join(os.path.split(chembl_beaker.__file__)[0], 'static')
PARAM_REGEX = re.compile(r'<[^<>]+>')
DEFAULT_APPS = [
    "chembl_beaker.beaker",
    "chembl_beaker.beaker.core_apps.calculations",
    "chembl_beaker.beaker.core_apps.conversions",
    "chembl_beaker.beaker.core_apps.descriptors",
    "chembl_beaker.beaker.core_apps.fingerprints",
    "chembl_beaker.beaker.core_apps.marvin",
    "chembl_beaker.beaker.core_apps.mcs",
    "chembl_beaker.beaker.core_apps.osra",
    "chembl_beaker.beaker.core_apps.rasterImages",
    "chembl_beaker.beaker.core_apps.svgImages",
    "chembl_beaker.beaker.core_apps.jsonImages",
    "chembl_beaker.beaker.core_apps.standarisation",
    "chembl_beaker.beaker.core_apps.D3Coords",
    "chembl_beaker.beaker.core_apps.similarityMaps",
    "chembl_beaker.beaker.core_apps.autoDocs",
    ]

#-----------------------------------------------------------------------------------------------------------------------

app = Bottle()
config = app.config

#-----------------------------------------------------------------------------------------------------------------------

if not getattr(config, 'load_config'):

        py = sys.version_info
        py3k = py >= (3, 0, 0)

        if py3k:
            from configparser import ConfigParser
        else:
            from ConfigParser import SafeConfigParser as ConfigParser

        from bottle import ConfigDict

        def load_config(self, filename):
            ''' Load values from an *.ini style config file.

                If the config file contains sections, their names are used as
                namespaces for the values within. The two special sections
                ``DEFAULT`` and ``bottle`` refer to the root namespace (no prefix).
            '''
            conf = ConfigParser()
            conf.read(filename)
            for section in conf.sections():
                for key, value in conf.items(section):
                    if section not in ('DEFAULT', 'bottle'):
                        key = section + '.' + key
                    self[key] = value
            return self

        ConfigDict.load_config = load_config

#-----------------------------------------------------------------------------------------------------------------------

apps = config.get('INSTALLED_APPS', None)
if apps:
    apps = json.loads(apps)
else:
    apps = DEFAULT_APPS

for module in apps:
    try:
        __import__(module + ".views")
    except Exception as e:
        print "Loading module %s failed because of error: %s" % (module, e.message)
        continue

#-----------------------------------------------------------------------------------------------------------------------




__author__ = 'mnowotka'

#-----------------------------------------------------------------------------------------------------------------------

from chembl_beaker.beaker import app
from chembl_beaker.beaker.core_apps.fingerprints.impl import _sdf2fps
from bottle import request
import base64
import json

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/sdf2fps/<ctab>', method=['OPTIONS', 'GET'], name="sdf2fps")
@app.route('/sdf2fps/<ctab>/<type>', method=['OPTIONS', 'GET'], name="sdf2fps")
def sdf2fps(ctab, type='morgan'):
    """
Computes fingerprints for given compounds and writes them as fps format. Type is a optional type of fingerprints,
can be 'morgan', 'pair' or 'maccs' (default to 'morgan'). CTAB is urlsafe_base64 encoded string containing single
molfile or concatenation of multiple molfiles.
    """
    sdf = base64.urlsafe_b64decode(ctab)
    return _sdf2fps(sdf, type)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/sdf2fps', method=['OPTIONS', 'POST'], name="sdf2fps")
def sdf2fps():
    """
Computes fingerprints for given compounds and writes them as fps format. Type is a optional type of fingerprints,
can be 'morgan', 'pair' or 'maccs' (default to 'morgan'). CTAB is either single molfile or SDF file.
    """

    params = json.loads(request.body.read())
    structure = params['structure']
    type = params.get('type', 'morgan')
    radius = params.get('radius', 2)
    n_bits = params.get('n_bits', 2048)
    return _sdf2fps(structure, type, radius, n_bits)

#-----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

from chembl_beaker.beaker import app
from chembl_beaker.beaker.core_apps.descriptors.impl import _getNumAtoms, _getLogP, _getTPSA, _getMolWt, _getDescriptors
from bottle import request
import base64
import json

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/getNumAtoms/<ctab>', method=['OPTIONS', 'GET'], name="getNumAtoms")
def getNumAtoms(ctab):
    """
Counts number of atoms of given compounds. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
    """

    data = base64.urlsafe_b64decode(ctab)
    return json.dumps(_getNumAtoms(data))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/getNumAtoms', method=['OPTIONS', 'POST'], name="getNumAtoms")
def getNumAtoms():
    """
Counts number of atoms of given compounds. CTAB is either single molfile or SDF file.
    """

    data = request.body.read()
    return json.dumps(_getNumAtoms(data))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/logP/<ctab>', method=['OPTIONS', 'GET'], name="logP")
def logP(ctab):
    """
Returns the logP value for a molecule. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
    """

    data = base64.urlsafe_b64decode(ctab)
    return json.dumps(_getLogP(data))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/logP', method=['OPTIONS', 'POST'], name="logP")
def logP():
    """
Returns the logP value for a molecule. CTAB is either single molfile or SDF file.
    """

    data = request.body.read()
    return json.dumps(_getLogP(data))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/tpsa/<ctab>', method=['OPTIONS', 'GET'], name="tpsa")
def tpsa(ctab):
    """
Returns the TPSA value for a molecule. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
    """

    data = base64.urlsafe_b64decode(ctab)
    return json.dumps(_getTPSA(data))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/tpsa', method=['OPTIONS', 'POST'], name="tpsa")
def tpsa():
    """
Returns the TPSA value for a molecule. CTAB is either single molfile or SDF file.
    """

    data = request.body.read()
    return json.dumps(_getTPSA(data))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/molWt/<ctab>', method=['OPTIONS', 'GET'], name="molWt")
def molWt(ctab):
    """
Returns molecular weight of a compound. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
    """

    data = base64.urlsafe_b64decode(ctab)
    return json.dumps(_getMolWt(data))
#-----------------------------------------------------------------------------------------------------------------------

@app.route('/molWt', method=['OPTIONS', 'POST'], name="molWt")
def molWt():
    """
Returns molecular weight of a compound. CTAB is either single molfile or SDF file.
    """

    data = request.body.read()
    return json.dumps(_getMolWt(data))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/descriptors/<ctab>', method=['OPTIONS', 'GET'], name="descriptors")
def descriptors(ctab):
    """
Returns descriptors of a compound. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
    """

    data = base64.urlsafe_b64decode(ctab)
    return json.dumps(_getDescriptors(data))
#-----------------------------------------------------------------------------------------------------------------------

@app.route('/descriptors', method=['OPTIONS', 'POST'], name="descriptors")
def descriptors():
    """
Returns descriptors of a compound. CTAB is either single molfile or SDF file.
    """

    data = request.body.read()
    return json.dumps(_getDescriptors(data, request.params))

#-----------------------------------------------------------------------------------------------------------------------
